@@ -77,9 +77,9 @@ class ClassConstraintComponent(ConstraintComponent):
 
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[Literal]:
         if len(self.class_rules) < 2:
-            m = "Value does not have class {}".format(stringify_node(datagraph, self.class_rules[0]))
+            m = "Value does not have class {}".format(stringify_node(self.shape.sg.graph, self.class_rules[0]))
         else:
-            rules = ", ".join(stringify_node(datagraph, c) for c in self.class_rules)
+            rules = ", ".join(stringify_node(self.shape.sg.graph, c) for c in self.class_rules)
             m = "Value class is not in classes ({})".format(rules)
         return [Literal(m)]
 
@@ -165,7 +165,7 @@ class DatatypeConstraintComponent(ConstraintComponent):
         return SH_DatatypeConstraintComponent
 
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[Literal]:
-        m = "Value is not Literal with datatype {}".format(stringify_node(datagraph, self.datatype_rule))
+        m = "Value is not Literal with datatype {}".format(stringify_node(self.shape.sg.graph, self.datatype_rule))
         return [Literal(m)]
 
     def evaluate(self, target_graph: GraphLike, focus_value_nodes: Dict, _evaluation_path: List):
@@ -258,7 +258,7 @@ class NodeKindConstraintComponent(ConstraintComponent):
         return SH_NodeKindConstraintComponent
 
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[Literal]:
-        m = "Value is not of Node Kind {}".format(stringify_node(datagraph, self.nodekind_rule))
+        m = "Value is not of Node Kind {}".format(stringify_node(self.shape.sg.graph, self.nodekind_rule))
         return [Literal(m)]
 
     def evaluate(self, target_graph: GraphLike, focus_value_nodes: Dict, _evaluation_path: List):
