@@ -26,6 +26,7 @@ SH_Literal = SH.term('Literal')
 SH_BlankNodeOrIRI = SH.term('BlankNodeOrIRI')
 SH_BlankNodeORLiteral = SH.term('BlankNodeOrLiteral')
 SH_IRIOrLiteral = SH.term('IRIOrLiteral')
+SH_SHACLFunction = SH.term('SHACLFunction')
 SH_SPARQLFunction = SH.term('SPARQLFunction')
 SH_SPARQLRule = SH.term('SPARQLRule')
 SH_TripleRule = SH.term('TripleRule')
@@ -40,6 +41,7 @@ RDF_object = RDF.term('object')
 RDF_predicate = RDF.term('predicate')
 RDF_subject = RDF.term('subject')
 RDFS_subClassOf = RDFS.term('subClassOf')
+RDFS_comment = RDFS.term('comment')
 SH_path = SH.term('path')
 SH_deactivated = SH.term('deactivated')
 SH_message = SH.term('message')
@@ -79,5 +81,10 @@ SH_subject = SH.term('subject')
 SH_predicate = SH.term('predicate')
 SH_object = SH.term('object')
 SH_parameter = SH.term('parameter')
+SH_ask = SH.term('ask')
 SH_select = SH.term('select')
 SH_this = SH.term('this')
+SH_filterShape = SH.term('filterShape')
+SH_nodes = SH.term('nodes')
+SH_union = SH.term('union')
+SH_intersection = SH.term('intersection')
